/***************************************************************************
 *            Core.cc
 *
 *  Copyright  2008  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
 
#include "Lince/Core.h"

using namespace Lince;

Core* Core::self = NULL;
SessionManagerPtr Core::ssm = SessionManagerPtr();
TorrentsManagerPtr Core::tm = TorrentsManagerPtr();
AlertsManagerPtr Core::am = AlertsManagerPtr();
SetupManagerPtr Core::sm = SetupManagerPtr();


Core::Core()
{
	get_session_manager();
}

void Core::init()
{
	static bool creating = false;
	if (!self && !creating)
		{
			creating = true;
			self = new Core();
			creating = false;
		}
}

void Core::uninit()
{
	ssm = SessionManagerPtr();
	tm = TorrentsManagerPtr();
	am = AlertsManagerPtr();
	sm = SetupManagerPtr();
	
	delete self;
}

AlertsManagerPtr Core::get_alerts_manager()
{
	if (!am)
		am = AlertsManager::create();

	return am;
}

SessionManagerPtr Core::get_session_manager()
{
	if (!ssm)
		ssm = SessionManager::create();
	
	return ssm;
}

SetupManagerPtr Core::get_setup_manager()
{
	if (!sm)
		sm = SetupManager::create();
	
	return sm;
}

TorrentsManagerPtr Core::get_torrents_manager()
{
	if (!tm)
		tm = TorrentsManager::create();
	
	return tm;
}

Core::~Core()
{
}
