//           EditTrackersDialog.cc
//  Tue Aug 12 23:02:18 2008
//  Copyright  2008  Fernando Tarín Morales
//  <icemanf@gmail.com>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

#include "EditTrackersDialog.h"

#include <gtkmm/scrolledwindow.h>
#include <gtkmm/box.h>
#include <gtkmm/button.h>

#include <glibmm/i18n.h>

using namespace Lince;

EditTrackersDialog::EditTrackersDialog(Torrent *_t):
	Gtk::Dialog("Trackers")
{
	t = _t;
	Gtk::ScrolledWindow *scrolledWindow = Gtk::manage( new Gtk::ScrolledWindow () );
	Gtk::HBox *hbox = Gtk::manage ( new Gtk::HBox ());
	Gtk::Button *addButton, *removeButton;
	addButton = Gtk::manage ( new Gtk::Button(_("Add")) );
	removeButton = Gtk::manage ( new Gtk::Button(_("Remove")) );
	
	hbox->pack_start(*addButton);
	hbox->pack_start(*removeButton);	
	
	m_refTreeModel = Gtk::ListStore::create(m_Columns);	
	m_TreeView.set_model(m_refTreeModel);
	m_TreeView.append_column("Url", m_Columns.m_col_name);
	
	scrolledWindow->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	scrolledWindow->add(m_TreeView);
	get_vbox()->set_homogeneous(false);
	get_vbox()->pack_start(*scrolledWindow, Gtk::PACK_EXPAND_WIDGET);
	get_vbox()->pack_start(*hbox, Gtk::PACK_SHRINK, 5);
	get_vbox()->show_all();
	
	add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
	
	addTrackers();
	
	has_been_modified = false;
	
	// This is used to get a reference to ok button.
	Gtk::Button *ok_button = static_cast<Gtk::Button *>(get_action_area()->children().front().get_widget());
	
	// Signals
	addButton->signal_clicked().connect( sigc::mem_fun(*this, &EditTrackersDialog::addTracker) );
	removeButton->signal_clicked().connect( sigc::mem_fun(*this, &EditTrackersDialog::removeTracker) );
	ok_button->signal_clicked().connect( sigc::mem_fun(*this, &EditTrackersDialog::exit) );
	
	int w, h;
	get_size(w, h);
	resize(w*4, h*2);
	
	run();
}

void EditTrackersDialog::addTrackers()
{
	int j=0;
	std::vector<libtorrent::announce_entry> trackers = t->trackers();
	std::vector<libtorrent::announce_entry>::iterator iter;
	
	m_refTreeModel->clear();
	
	for (iter = trackers.begin(); iter != trackers.end(); ++iter){
		Gtk::TreeModel::Row row = *(m_refTreeModel->append());
  		row[m_Columns.m_col_id] = j;
  		row[m_Columns.m_col_name] = iter->url;
		j++;
	}
}

void EditTrackersDialog::addTracker()
{
	Glib::ustring url;
	
	if(Utils::askUstringDialog(_("Enter the tracker url"), _("Enter the tracker url to add it to the tracker's list"), url)){
		if (!Utils::isAnURL(url)){
			Utils::warningDialog(_("Tracker error"), _("Introduced tracker is not an url"));
			return;			
		}
		std::vector<libtorrent::announce_entry> trackers = t->trackers();
		libtorrent::announce_entry tracker(url);		
		trackers.push_back(tracker);		
		t->replace_trackers(trackers);
		
		Gtk::TreeModel::Row row = *(m_refTreeModel->append());
  		row[m_Columns.m_col_id] = m_refTreeModel->children().size()-1;
  		row[m_Columns.m_col_name] = url;
		
		has_been_modified = true;
	}
}

void EditTrackersDialog::removeTracker()
{
	Gtk::TreeModel::iterator iter;
	iter = m_TreeView.get_selection()->get_selected();
	if (iter)
	{
		if (m_refTreeModel->children().size() == 1)
		{
			Utils::warningDialog(_("Tracker error"), _("At least one tracker is needed"));
			return;
		}
		std::vector<libtorrent::announce_entry> trackers;
		m_refTreeModel->erase(iter);
		for (iter = m_refTreeModel->children().begin(); iter != m_refTreeModel->children().end(); ++iter)
		{
			Glib::ustring url = iter->get_value(m_Columns.m_col_name);
			libtorrent::announce_entry tracker(url);
			trackers.push_back(tracker);
		}
		
		has_been_modified = true;
		t->replace_trackers(trackers);
		
		addTrackers();
	}
}

void EditTrackersDialog::exit(){
	if (has_been_modified)
		t->force_reannounce();
}

EditTrackersDialog::~EditTrackersDialog()
{
}
