/***************************************************************************
 *            Notifications.cc
 *
 *  Copyright  2009  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#include "Notifications.h"
#include "../config.h"

#include "Lince/Core.h"
#include "Lince/AlertsManager.h"
#include "Lince/Utils.h"

#include <libnotify/notify.h>

#include <glibmm/main.h>

using namespace Lince;

Notifications::Notifications(SystemTray *st)
{
	show_notifications = true;
	show_from_systray = true;
	init_period = true;

	for (int i=firstMessageType; i<lastMessageType; i++)
		notify_message[i] = false;

	gtkStatusIcon = st->getGtkStatusIcon();

	notify_init("Lince");

	Glib::ustring logoFile = Glib::build_filename(ICON_DIR, "lince.png");
	// Icon
	logo_icon = Gdk::Pixbuf::create_from_file(logoFile);

	Core::get_alerts_manager()->finished_torrent_signal_log(
			boost::bind(&Notifications::addNotification, this, _1, torrentFinished));

	Core::get_alerts_manager()->torrent_added_log_signal(
			boost::bind(&Notifications::addNotification, this, _1, torrentAdded));

	Core::get_alerts_manager()->torrent_removed_signal(
			boost::bind(&Notifications::addNotification, this, _1, torrentRemoved));

	Core::get_alerts_manager()->torrent_stopped_signal(
			boost::bind(&Notifications::addNotification, this, _1, torrentStopped));

	Core::get_alerts_manager()->torrent_started_signal(
			boost::bind(&Notifications::addNotification, this, _1, torrentStarted));

	Core::get_alerts_manager()->torrent_queued_signal(
			boost::bind(&Notifications::addNotification, this, _1, torrentQueued));

	Core::get_alerts_manager()->torrent_unqueued_signal(
			boost::bind(&Notifications::addNotification, this, _1, torrentUnqueued));

	Glib::signal_timeout().connect(sigc::mem_fun(*this, &Notifications::initPeriodFinished), 1500);
}

void Notifications::setNotifyMessage(int type, bool value)
{
	if (type < firstMessageType || type >= lastMessageType)
		return;

	notify_message[type] = value;
}

bool Notifications::notifyMessage(std::string &message)
{
	NotifyNotification *n;

    if (show_from_systray){
    	n = notify_notification_new_with_status_icon ("Lince", message.c_str(),
    	    			NULL, gtkStatusIcon);
    	//notify_notification_set_icon_from_pixbuf(n, logo_icon->gobj_copy() );
    }
    else{
    	n = notify_notification_new ("Lince", message.c_str(), NULL, NULL);
    	notify_notification_set_icon_from_pixbuf(n, logo_icon->gobj_copy() );
    }

    notify_notification_set_timeout (n, 5000); // 5 seconds

    if (!notify_notification_show (n, NULL))
		return false;

    g_object_unref(G_OBJECT(n));

    return true;
}

void Notifications::addNotification(std::string &message, int type)
{
	if (!notify_message[type] || !show_notifications || init_period)
			return;

	notifications.push(message);
}

void Notifications::flushNotification(void){
	if (notifications.empty())
		return;

	notifyMessage(notifications.front());
	notifications.pop();
}

bool Notifications::initPeriodFinished()
{
	init_period = false;
	return false;
}

Notifications::~Notifications()
{
}
