//           TorrentsTab.cc
//  Sat Feb 17 21:42:42 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

#include "TorrentsTab.h"
#include "MenuBar.h"

#include "Lince/Core.h"
#include "Lince/AlertsManager.h"

#include <iostream>

using namespace Lince;

TorrentsTab::TorrentsTab(MainWindow *m, MenuBar *mb):
	BaseTab("Torrents"),
	downloadedTorrents(m, mb, this, DOWNLOADED_TORRENTS),
	downloadingTorrents(m, mb, this, DOWNLOADING_TORRENTS)
{
	pane.pack1(downloadingTorrents, true, true);
	pane.pack2(downloadedTorrents, true, true);
	
	Core::get_alerts_manager()->torrent_added_signal(
			boost::bind(&TorrentsTab::addTorrentFromPosition, this, _1));
	
	Core::get_torrents_manager()->switched_torrent_signal(
			boost::bind(&TorrentsTab::switchTorrent, this, _1));
}

void TorrentsTab::switchTorrent(int position)
{
	Torrent *t;
	int last_position = Core::get_torrents_manager()->shared_torrents_count();
		
	downloadingTorrents.removeGivenRow(position);
	downloadingTorrents.updateRows();
	
	t = Core::get_torrents_manager()->get_shared_torrent_at(last_position);
	
	downloadedTorrents.addRow(t);
}

void TorrentsTab::checkTorrentsSelection(TorrentsWindowType t)
{
	if ( t == DOWNLOADING_TORRENTS)
		downloadedTorrents.unselectRows();
	else
		downloadingTorrents.unselectRows();
}

torrentPosition TorrentsTab::getSelectedTorrentPosition()
{
	int torrentId=0;
	bool isDownloaded;
	
	if (getDownloadingSelectedTorrents())
	{
		downloadingTorrents.getSelectedTorrent(torrentId);
		isDownloaded = false;
	}
	else if (getDownloadedSelectedTorrents())
	{
		downloadedTorrents.getSelectedTorrent(torrentId);
		isDownloaded = true;
	}
	else
		torrentId = 0;

	return torrentPosition(std::make_pair(torrentId, isDownloaded));
}

std::vector<torrentPosition> TorrentsTab::getSelectedTorrentsPositions()
{
	std::vector<int> torrentsIds;
	std::vector<torrentPosition> torrentsPositions;
	bool isDownloaded;
	
	if (getDownloadingSelectedTorrents())
	{
		downloadingTorrents.getSelectedTorrents(torrentsIds);
		isDownloaded = false;
	}
	else if (getDownloadedSelectedTorrents())
	{
		downloadedTorrents.getSelectedTorrents(torrentsIds);
		isDownloaded = true;
	}
	else
		return torrentsPositions;
	
	for (std::vector<int>::iterator i=torrentsIds.begin(); i != torrentsIds.end(); ++i)
	{
		torrentsPositions.push_back(std::make_pair(*i, isDownloaded));
	}
	
	return torrentsPositions;
}


void TorrentsTab::addTorrentFromPosition(torrentPosition tp)
{
	TorrentsManagerPtr tm = Core::get_torrents_manager();
	
	if (!tp.second)
	{
		downloadingTorrents.addRow(tm->get_torrent_at(tp));
		updateDownloadingTorrents();
	}
	else
	{
		downloadedTorrents.addRow(tm->get_torrent_at(tp));
		updateDownloadedTorrents();
	}
}

TorrentsTab::~TorrentsTab()
{
}

	
