/***************************************************************************
 *            TorrentsWindow.cc
 *
 *  Sat Jan 27 14:19:08 2007
 *  Copyright  2007  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#include "TorrentsWindow.h"
#include "TorrentsTab.h"
#include "MainWindow.h"
#include "XMLReader.h"
#include "XMLWriter.h"

#include "../config.h"
#include <glibmm/i18n.h>

#include "Lince/Core.h"
#include "Lince/TorrentsManager.h"
#include "Lince/Utils.h"

#include "libtorrent/entry.hpp"
#include "libtorrent/bencode.hpp"

#include <sstream>

using namespace Lince;

TorrentsWindow::TorrentsWindow(MainWindow *m, MenuBar *mb, TorrentsTab *t, TorrentsWindowType torrWinType):
	ScrolledWindow(),
	rm_pMenuPopup(0),
	nsm_pMenuPopup(0)
{
	torrentsWindowType = torrWinType;
	menuBar = mb;
	torrentsTab = t;

	// Add the TreeView, inside the ScrolledWindow, with the button underneath:
	// Only show the scrollbars when they are necessary:
	add(torrentsTreeView);
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	// Create the Tree model:
	refTreeModel = Gtk::ListStore::create(torrentsColumns);
	torrentsTreeView.set_model(refTreeModel);
	torrentsTreeView.set_rules_hint();
	refTreeSelection = torrentsTreeView.get_selection();
	refTreeSelection->set_mode(Gtk::SELECTION_MULTIPLE);

	// Add the TreeView's view columns:
	torrentsTreeView.append_column("ID", torrentsColumns.m_col_id); // This number will be shown with the default m_TreeView.append_column("Name", m_Columns.m_col_name);
	torrentsTreeView.append_column(_("Health"), torrentsColumns.m_col_health);
	torrentsTreeView.append_column(_("Name"), torrentsColumns.m_col_name);
	torrentsTreeView.append_column(_("Size"), torrentsColumns.m_col_size);

	Gtk::CellRendererProgress* cell = Gtk::manage(new Gtk::CellRendererProgress);
	int cols_count = torrentsTreeView.append_column(_("Percentage"), *cell);

	Gtk::TreeViewColumn* pColumn = torrentsTreeView.get_column(cols_count-1);
	if(pColumn)
	{
#ifdef GLIBMM_PROPERTIES_ENABLED
				pColumn->add_attribute(cell->property_value(), torrentsColumns.m_col_percentage);
#else
				pColumn->add_attribute(*cell, "value", torrentsColumns.m_col_percentage);
#endif
	}

	torrentsTreeView.append_column(_("Rest"), torrentsColumns.m_col_rest);
	torrentsTreeView.append_column(_("Downloaded"), torrentsColumns.m_col_downloaded);
	torrentsTreeView.append_column(_("Uploaded"), torrentsColumns.m_col_uploaded);
	torrentsTreeView.append_column(_("DL Speed"), torrentsColumns.m_col_download_speed);
	torrentsTreeView.append_column(_("UL Speed"), torrentsColumns.m_col_upload_speed);
	torrentsTreeView.append_column(_("Ratio"), torrentsColumns.m_col_ratio);
	torrentsTreeView.append_column(_("Seeders"), torrentsColumns.m_col_seeders);
	torrentsTreeView.append_column(_("Downloaders"), torrentsColumns.m_col_downloaders);
	torrentsTreeView.append_column(_("Status"), torrentsColumns.m_col_status);
	torrentsTreeView.append_column("ETA", torrentsColumns.m_col_time);

	//torrentsTreeView.signal_row_activated().connect(sigc::mem_fun(*this, &TorrentsWindow::funcionPrueba));
	//torrentsTreeView.signal_button_press_event().connect(sigc::mem_fun(*this,  &TorrentsWindow::on_button_press_event1));
	torrentsTreeView.signal_row_activated().connect(sigc::bind<Glib::RefPtr<Gtk::ListStore> > (sigc::mem_fun(*m, &MainWindow::add_tinfo_tab), refTreeModel));
	torrentsTreeView.signal_cursor_changed().connect(sigc::bind<TorrentsWindowType> (sigc::mem_fun(*t, &TorrentsTab::checkTorrentsSelection), torrentsWindowType));
	refTreeSelection->signal_changed().connect(sigc::mem_fun(*this, &TorrentsWindow::setMenuSensitivity));
	torrentsTreeView.signal_button_press_event().connect_notify(sigc::mem_fun(*this,  &TorrentsWindow::mouse_button_press_event));
	torrentsTreeView.signal_key_press_event().connect_notify(sigc::mem_fun(*this,  &TorrentsWindow::key_press_event));
	torrentsTreeView.signal_key_release_event().connect_notify(sigc::mem_fun(*this,  &TorrentsWindow::key_release_event));
	torrentsTreeView.signal_focus_in_event().connect(sigc::mem_fun(*this,  &TorrentsWindow::focus_in_event));

	for(guint i = 0; i < 15; i++)
	{
		Gtk::TreeView::Column* pColumn = torrentsTreeView.get_column(i);
		Gtk::CellRenderer* cColumn = torrentsTreeView.get_column_cell_renderer(i);

		//pColumn->set_reorderable();

		pColumn->set_resizable();
		pColumn->set_clickable();

		// Alignment | sortable
		if (i == 2)
		{
			pColumn->set_sort_column(torrentsColumns.m_col_name);
			pColumn->set_alignment(0);
			cColumn->property_xalign() = 0.0;
		}
		else if (i == 4)
		{
			pColumn->set_sort_column(torrentsColumns.m_col_percentage);
			pColumn->set_alignment(0.5);
			pColumn->set_expand();
			cColumn->property_xalign() = 0.0;
		}
		else
		{
			pColumn->set_alignment(1.0);
			cColumn->property_xalign() = 1.0;
			switch(i)
			{
				case 0:
					pColumn->set_sort_column(torrentsColumns.m_col_id);
					break;
				case 1:
					pColumn->set_sort_column(torrentsColumns.m_health);
					break;
				case 3:
					pColumn->set_sort_column(torrentsColumns.m_size);
					break;
				case 5:
					pColumn->set_sort_column(torrentsColumns.m_rest);
					break;
				case 6:
					pColumn->set_sort_column(torrentsColumns.m_downloaded);
					break;
				case 7:
					pColumn->set_sort_column(torrentsColumns.m_uploaded);
					break;
				case 8:
					pColumn->set_sort_column(torrentsColumns.m_down_speed);
					break;
				case 9:
					pColumn->set_sort_column(torrentsColumns.m_up_speed);
					break;
				case 10:
					pColumn->set_sort_column(torrentsColumns.m_ratio);
					break;
				case 11:
					pColumn->set_sort_column(torrentsColumns.m_col_seeders);
					break;
				case 12:
					pColumn->set_sort_column(torrentsColumns.m_col_downloaders);
					break;
				case 13:
					pColumn->set_sort_column(torrentsColumns.m_col_status);
					break;
				case 14:
					pColumn->set_sort_column(torrentsColumns.m_time);
					break;
			}
		}
	}

	torrentsTreeView.columns_autosize();

	// Menu actions
	m_refActionGroup = Gtk::ActionGroup::create();
	m_refActionGroup->add( Gtk::Action::create("ContextMenu", "Context Menu"));
	m_refActionGroup->add( Gtk::Action::create("ShowDetails", _("_Show Details"), "_Show info"), sigc::mem_fun(m, &MainWindow::add_tinfo_tab_from_menu) );

	m_refActionGroup->add( Gtk::Action::create("Speed", _("_Speed")) );

	m_refActionGroup->add( Gtk::Action::create("Upload", _("Upload")) );
	m_refActionGroup->add( Gtk::Action::create("Unlimitedus", _("Unlimited"), "Unlimited"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_speed), -1) );
	m_refActionGroup->add( Gtk::Action::create("20us", "20 KB", "20 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_speed), 20) );
	m_refActionGroup->add( Gtk::Action::create("40us", "40 KB", "40 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_speed), 40) );
	m_refActionGroup->add( Gtk::Action::create("60us", "60 KB", "60 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_speed), 60) );
	m_refActionGroup->add( Gtk::Action::create("80us", "80 KB", "80 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_speed), 80) );
	m_refActionGroup->add( Gtk::Action::create("100us", "100 KB", "100 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_speed), 100) );
	m_refActionGroup->add( Gtk::Action::create("200us", "200 KB", "200 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_speed), 200) );
	m_refActionGroup->add( Gtk::Action::create("400us", "400 KB", "400 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_speed), 400) );
	m_refActionGroup->add( Gtk::Action::create("Manualus", _("Manual"), "Manual"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_speed), 0) );


	m_refActionGroup->add( Gtk::Action::create("Download", _("Download")) );
	m_refActionGroup->add( Gtk::Action::create("Unlimitedds", _("Unlimited"), "Unlimited"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_download_speed), -1) );
	m_refActionGroup->add( Gtk::Action::create("20ds", "20 KB", "20 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_download_speed), 20) );
	m_refActionGroup->add( Gtk::Action::create("40ds", "40 KB", "40 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_download_speed), 40) );
	m_refActionGroup->add( Gtk::Action::create("60ds", "60 KB", "60 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_download_speed), 60) );
	m_refActionGroup->add( Gtk::Action::create("80ds", "80 KB", "80 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_download_speed), 80) );
	m_refActionGroup->add( Gtk::Action::create("100ds", "100 KB", "100 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_download_speed), 100) );
	m_refActionGroup->add( Gtk::Action::create("200ds", "200 KB", "200 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_download_speed), 200) );
	m_refActionGroup->add( Gtk::Action::create("400ds", "400 KB", "400 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_download_speed), 400) );
	m_refActionGroup->add( Gtk::Action::create("Manualds", _("Manual"), "Manual"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_download_speed), 0) );

	m_refActionGroup->add( Gtk::Action::create("Connections", _("_Connections")) );

	m_refActionGroup->add( Gtk::Action::create("MaxConnections", _("Max Connections")) );
	m_refActionGroup->add( Gtk::Action::create("Disabledmc", _("Disable"), "_Disable"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_max_connections), -1) );
	m_refActionGroup->add( Gtk::Action::create("10mc", "10", "10"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_max_connections), 10) );
	m_refActionGroup->add( Gtk::Action::create("20mc", "20", "20"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_max_connections), 20) );
	m_refActionGroup->add( Gtk::Action::create("40mc", "40", "40"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_max_connections), 40) );
	m_refActionGroup->add( Gtk::Action::create("80mc", "80", "80"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_max_connections), 80) );
	m_refActionGroup->add( Gtk::Action::create("100mc", "100", "100"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_max_connections), 100) );
	m_refActionGroup->add( Gtk::Action::create("Manualmc", _("Manual"), "Manual"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_max_connections), 0) );

	m_refActionGroup->add( Gtk::Action::create("UploadSlots", _("Upload Slots")) );
	m_refActionGroup->add( Gtk::Action::create("Disabledul", _("Disable"), "_Disable"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_slots), -1) );
	m_refActionGroup->add( Gtk::Action::create("3ul", "3", "3"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_slots), 3) );
	m_refActionGroup->add( Gtk::Action::create("6ul", "6", "6"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_slots), 6) );
	m_refActionGroup->add( Gtk::Action::create("12ul", "12", "12"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_slots), 12) );
	m_refActionGroup->add( Gtk::Action::create("24ul", "24", "24"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_slots), 24) );
	m_refActionGroup->add( Gtk::Action::create("Manualul", _("Manual"), "Manual"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_torrent_upload_slots), 0) );

	m_refActionGroup->add( Gtk::Action::create("ShareRatio", _("_Share Ratio")) );
	m_refActionGroup->add( Gtk::Action::create("Disabledsr", _("Disable"), "_Disable"), sigc::bind<double> (sigc::mem_fun(m, &MainWindow::set_torrent_share_ratio), 0) );
	m_refActionGroup->add( Gtk::Action::create("1sr", "1", "1"), sigc::bind<double> (sigc::mem_fun(m, &MainWindow::set_torrent_share_ratio), 1) );
	m_refActionGroup->add( Gtk::Action::create("1.5sr", "1.5", "1.5"), sigc::bind<double> (sigc::mem_fun(m, &MainWindow::set_torrent_share_ratio), 1.5) );
	m_refActionGroup->add( Gtk::Action::create("2sr", "2", "2"), sigc::bind<double> (sigc::mem_fun(m, &MainWindow::set_torrent_share_ratio), 2) );
	m_refActionGroup->add( Gtk::Action::create("2.5sr", "2.5", "2.5"), sigc::bind<double> (sigc::mem_fun(m, &MainWindow::set_torrent_share_ratio), 2.5) );
	m_refActionGroup->add( Gtk::Action::create("3sr", "3", "3"), sigc::bind<double> (sigc::mem_fun(m, &MainWindow::set_torrent_share_ratio), 3) );
	m_refActionGroup->add( Gtk::Action::create("3.5sr", "3.5", "3.5"), sigc::bind<double> (sigc::mem_fun(m, &MainWindow::set_torrent_share_ratio), 3.5) );
	m_refActionGroup->add( Gtk::Action::create("4sr", "4", "4"), sigc::bind<double> (sigc::mem_fun(m, &MainWindow::set_torrent_share_ratio), 4) );

	m_refActionGroup->add( Gtk::Action::create("Tracker", "Tracker") );
	m_refActionGroup->add( Gtk::Action::create("EditTrackers", _("Edit trackers")), sigc::mem_fun(m, &MainWindow::edit_torrent_trackers) );
	m_refActionGroup->add( Gtk::Action::create("TrackerLoginData", _("Set Tracker log-in")), sigc::mem_fun(m, &MainWindow::trackerLogin) );
	m_refActionGroup->add( Gtk::Action::create("ForceReannounce", _("Force Reannounce")), sigc::mem_fun(m, &MainWindow::torrent_force_reannounce) );

	m_refActionGroup->add( Gtk::Action::create("TorrentF", _("Torrent")) );
	m_refActionGroup->add( Gtk::Action::create("ExportTorrent", _("Export Torrent")), sigc::mem_fun(m, &MainWindow::export_torrent) );

	m_refActionGroup->add( Gtk::Action::create("Resume", _("Resume"), "Resume"), sigc::mem_fun(m, &MainWindow::resume_torrents) );
	m_refActionGroup->add( Gtk::Action::create("Stop", _("Stop"), "Stop"), sigc::mem_fun(m, &MainWindow::stop_torrents) );
	m_refActionGroup->add( Gtk::Action::create("Remove", _("Remove"), "Remove"), sigc::mem_fun(m, &MainWindow::remove_torrents) );
	m_refActionGroup->add( Gtk::Action::create("ForceRecheck", _("Force Recheck"), "Force Recheck"), sigc::mem_fun(m, &MainWindow::recheck_torrent) );

	m_refActionGroup->add( Gtk::Action::create("ResumeAll", _("Resume all"), "Resume All"), sigc::mem_fun(m, &MainWindow::start_all_torrents) );
	m_refActionGroup->add( Gtk::Action::create("StopAll", _("Stop all"), "Stop All"), sigc::mem_fun(m, &MainWindow::stop_all_torrents) );

	m_refActionGroup->add( Gtk::Action::create("Columns", _("_Columns")) );

	showID = Gtk::ToggleAction::create("ShowID", _("Show ID"), "ID");
	showID->set_active();
	m_refActionGroup->add( showID, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 0));

	showHealth = Gtk::ToggleAction::create("ShowHealth", _("Show Health"), "Health");
	showHealth->set_active();
	m_refActionGroup->add( showHealth, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 1));

	showName = Gtk::ToggleAction::create("ShowName", _("Show Name"), "Name");
	showName->set_active();
	m_refActionGroup->add( showName, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 2));

	showSize = Gtk::ToggleAction::create("ShowSize", _("Show Size"), "Size");
	showSize->set_active();
	m_refActionGroup->add( showSize, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 3));

	showPercentage = Gtk::ToggleAction::create("ShowPercentage", _("Show Percentage"), "Percentage");
	showPercentage->set_active();
	m_refActionGroup->add( showPercentage, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 4));

	showRest = Gtk::ToggleAction::create("ShowRest", _("Show Rest"), "Rest");
	showRest->set_active();
	m_refActionGroup->add( showRest, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 5));

	showDownloaded = Gtk::ToggleAction::create("ShowDownloaded", _("Show Downloaded"), "Downloaded");
	showDownloaded->set_active();
	m_refActionGroup->add( showDownloaded, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 6));

	showUploaded = Gtk::ToggleAction::create("ShowUploaded", _("Show Uploaded"), "Uploaded");
	showUploaded->set_active();
	m_refActionGroup->add( showUploaded, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 7));

	showDSpeed = Gtk::ToggleAction::create("ShowDSpeed", _("Show Down. Speed"), "Down Speed");
	showDSpeed->set_active();
	m_refActionGroup->add( showDSpeed, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 8));

	showUSpeed = Gtk::ToggleAction::create("ShowUSpeed", _("Show Up. Speed"), "Up Speed");
	showUSpeed->set_active();
	m_refActionGroup->add( showUSpeed, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 9));

	showRatio = Gtk::ToggleAction::create("ShowRatio", _("Show Ratio"), "Ratio");
	showRatio->set_active();
	m_refActionGroup->add( showRatio, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 10));

	showSeeders = Gtk::ToggleAction::create("ShowSeeders", _("Show Seeders"), "Seeders");
	showSeeders->set_active();
	m_refActionGroup->add( showSeeders, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 11));

	showDownloaders = Gtk::ToggleAction::create("ShowDownloaders", _("Show Downloaders"), "Downloaders");
	showDownloaders->set_active();
	m_refActionGroup->add( showDownloaders, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 12));

	showStatus = Gtk::ToggleAction::create("ShowStatus", _("Show Status"), "Status");
	showStatus->set_active();
	m_refActionGroup->add( showStatus, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 13));

	showETA = Gtk::ToggleAction::create("ShowETA", _("Show ETA"), "ETA");
	showETA->set_active();
	m_refActionGroup->add( showETA, sigc::bind<int> (sigc::mem_fun(this, &TorrentsWindow::hideColumn), 14));

	rm_refUIManager = Gtk::UIManager::create();
	rm_refUIManager->insert_action_group(m_refActionGroup);

	Glib::ustring rm_ui_info =
    "<ui>"
    "  <popup name='PopupMenu'>"
    "    <menuitem action='ShowDetails'/>"
	"    <separator/>"
	"	 <menu action='Speed'>"
		"	 <menu action='Upload'>"
		"    <menuitem action='Unlimitedus'/>"
		"    <separator/>"
		"    <menuitem action='20us'/>"
		"    <menuitem action='40us'/>"
		"    <menuitem action='60us'/>"
		"    <menuitem action='80us'/>"
		"    <menuitem action='100us'/>"
		"    <menuitem action='200us'/>"
		"    <menuitem action='400us'/>"
		"    <separator/>"
		"    <menuitem action='Manualus'/>"
		"	 </menu>"
		"	 <menu action='Download'>"
		"    <menuitem action='Unlimitedds'/>"
		"    <separator/>"
		"    <menuitem action='20ds'/>"
		"    <menuitem action='40ds'/>"
		"    <menuitem action='60ds'/>"
		"    <menuitem action='80ds'/>"
		"    <menuitem action='100ds'/>"
		"    <menuitem action='200ds'/>"
		"    <menuitem action='400ds'/>"
		"    <separator/>"
		"    <menuitem action='Manualds'/>"
		"	 </menu>"
	"	 </menu>"
	"	 <menu action='Connections'>"
		"	 <menu action='MaxConnections'>"
		"    <menuitem action='Disabledmc'/>"
		"    <separator/>"
		"    <menuitem action='10mc'/>"
		"    <menuitem action='20mc'/>"
		"    <menuitem action='40mc'/>"
		"    <menuitem action='80mc'/>"
		"    <menuitem action='100mc'/>"
		"    <separator/>"
		"    <menuitem action='Manualmc'/>"
		"	 </menu>"
		"	 <menu action='UploadSlots'>"
		"    <menuitem action='Disabledul'/>"
		"    <separator/>"
		"    <menuitem action='3ul'/>"
		"    <menuitem action='6ul'/>"
		"    <menuitem action='12ul'/>"
		"    <menuitem action='24ul'/>"
		"    <separator/>"
		"    <menuitem action='Manualul'/>"
		"	 </menu>"
	"	 </menu>"
	"	 <menu action='ShareRatio'>"
		"    <menuitem action='Disabledsr'/>"
		"    <separator/>"
		"    <menuitem action='1sr'/>"
		"    <menuitem action='1.5sr'/>"
		"    <menuitem action='2sr'/>"
		"    <menuitem action='2.5sr'/>"
		"    <menuitem action='3sr'/>"
		"    <menuitem action='3.5sr'/>"
		"    <menuitem action='4sr'/>"
	"	 </menu>"
	"	 <menu action='Tracker'>"
		"    <menuitem action='EditTrackers'/>"
		"    <menuitem action='TrackerLoginData'/>"
		"    <menuitem action='ForceReannounce'/>"
	"	 </menu>"
	"	 <menu action='TorrentF'>"
		"    <menuitem action='ExportTorrent'/>"
	"	 </menu>"
	"    <separator/>"
	"    <menuitem action='Resume'/>"
	"    <menuitem action='Stop'/>"
	"    <menuitem action='Remove'/>"
	"    <menuitem action='ForceRecheck'/>"
	"    <separator/>"
	"    <menuitem action='ResumeAll'/>"
	"    <menuitem action='StopAll'/>"
	"    <separator/>"
	"	 <menu action='Columns'>"
	"    <menuitem action='ShowID'/>"
	"    <menuitem action='ShowHealth'/>"
	"    <menuitem action='ShowName'/>"
	"    <menuitem action='ShowSize'/>"
	"    <menuitem action='ShowPercentage'/>"
	"    <menuitem action='ShowDownloaded'/>"
	"    <menuitem action='ShowUploaded'/>"
	"    <menuitem action='ShowDSpeed'/>"
	"    <menuitem action='ShowUSpeed'/>"
	"    <menuitem action='ShowRatio'/>"
	"    <menuitem action='ShowRest'/>"
	"    <menuitem action='ShowSeeders'/>"
	"    <menuitem action='ShowDownloaders'/>"
	"    <menuitem action='ShowStatus'/>"
	"    <menuitem action='ShowETA'/>"
	"	 </menu>"
    "  </popup>"
    "</ui>";


	nsm_refUIManager = Gtk::UIManager::create();
	nsm_refUIManager->insert_action_group(m_refActionGroup);

	Glib::ustring nsm_ui_info =
	"<ui>"
    "  <popup name='PopupMenu'>"
	"    <menuitem action='ResumeAll'/>"
	"    <menuitem action='StopAll'/>"
	"    <separator/>"
	"	 <menu action='Columns'>"
	"    <menuitem action='ShowID'/>"
	"    <menuitem action='ShowHealth'/>"
	"    <menuitem action='ShowName'/>"
	"    <menuitem action='ShowSize'/>"
	"    <menuitem action='ShowPercentage'/>"
	"    <menuitem action='ShowDownloaded'/>"
	"    <menuitem action='ShowUploaded'/>"
	"    <menuitem action='ShowDSpeed'/>"
	"    <menuitem action='ShowUSpeed'/>"
	"    <menuitem action='ShowRatio'/>"
	"    <menuitem action='ShowRest'/>"
	"    <menuitem action='ShowSeeders'/>"
	"    <menuitem action='ShowDownloaders'/>"
	"    <menuitem action='ShowStatus'/>"
	"    <menuitem action='ShowETA'/>"
	"	 </menu>"
    "  </popup>"
    "</ui>";

#ifdef GLIBMM_EXCEPTIONS_ENABLED
	try
	{
		rm_refUIManager->add_ui_from_string(rm_ui_info);
		nsm_refUIManager->add_ui_from_string(nsm_ui_info);
	}
	catch(const Glib::Error& ex)
	{
		std::cerr << "building menus failed: " <<  ex.what();
	}
#else
	std::auto_ptr<Glib::Error> ex;
	rm_refUIManager->add_ui_from_string(rm_ui_info, ex);
	if(ex.get())
	{
		std::cerr << "building menus failed: " <<  ex->what();
	}
	nsm_refUIManager->add_ui_from_string(nsm_ui_info, ex);
	if(ex.get())
	{
		std::cerr << "building menus failed: " <<  ex->what();
	}
#endif //GLIBMM_EXCEPTIONS_ENABLED

	rm_pMenuPopup = Gtk::manage(dynamic_cast<Gtk::Menu*>(rm_refUIManager->get_widget("/PopupMenu")));
	if(!rm_pMenuPopup)
		g_warning("menu not found");

	nsm_pMenuPopup = Gtk::manage(dynamic_cast<Gtk::Menu*>(nsm_refUIManager->get_widget("/PopupMenu")));
	if(!nsm_pMenuPopup)
		g_warning("menu not found");

	isCtrlKeyPressed = false;

	XMLReader reader;
	Glib::ustring fname;

	if (torrentsWindowType == DOWNLOADING_TORRENTS)
	{
		fname = Glib::build_filename(get_base_directory().native_directory_string(),
									 Glib::ustring("downloadcolumns.xml"));
	}
	else
	{
		fname = Glib::build_filename(get_base_directory().native_directory_string(),
									 Glib::ustring("finishedcolumns.xml"));
	}

	reader.setFileName(fname);

	if (reader.goToRootElement())
	{
		reader.goToNextElement();
		reader.goToNextElement();
		showID->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showHealth->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showName->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showSize->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showDownloaded->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showUploaded->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showDSpeed->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showUSpeed->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showRatio->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showRest->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showPercentage->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showSeeders->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showDownloaders->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showStatus->set_active(to_int(reader.getChildData()));
		reader.goToNextElement();
		showETA->set_active(to_int(reader.getChildData()));
	}
	else if (torrentsWindowType == DOWNLOADED_TORRENTS)
	{
		showDownloaded->set_active(false);
		showDSpeed->set_active(false);
		showRest->set_active(false);
		showETA->set_active(false);
	}

	// Health
	goodHealth = Gdk::Pixbuf::create_from_file(Glib::build_filename(ICON_DIR, "connection-ok.png"));
	badHealth = Gdk::Pixbuf::create_from_file(Glib::build_filename(ICON_DIR, "no-connection.png"));
	firewalledHealth = Gdk::Pixbuf::create_from_file(Glib::build_filename(ICON_DIR, "firewalled.png"));
	noTrackerHealth = Gdk::Pixbuf::create_from_file(Glib::build_filename(ICON_DIR, "seeding.png"));
}

TorrentsWindow::~TorrentsWindow()
{
	XMLWriter writer;
	Glib::ustring fname;

	if (torrentsWindowType == DOWNLOADING_TORRENTS)
		fname = Glib::build_filename(get_base_directory().native_directory_string(),
									 Glib::ustring("downloadcolumns.xml"));
	else
		fname = Glib::build_filename(get_base_directory().native_directory_string(),
									 std::string("finishedcolumns.xml"));

	writer.setFileName(fname);
	writer.startDocument("Lince");
	if (torrentsWindowType == DOWNLOADING_TORRENTS)
		writer.startElement("DownloadColumns");
	else
		writer.startElement("FinishedColumns");

	writer.startElement("ShowID");
	writer.addAttribute("type", "int");
	writer.addElementData(showID->get_active());
	writer.endElement();

	writer.startElement("ShowHealth");
	writer.addAttribute("type", "int");
	writer.addElementData(showHealth->get_active());
	writer.endElement();

	writer.startElement("ShowName");
	writer.addAttribute("type", "int");
	writer.addElementData(showName->get_active());
	writer.endElement();

	writer.startElement("ShowSize");
	writer.addAttribute("type", "int");
	writer.addElementData(showSize->get_active());
	writer.endElement();

	writer.startElement("ShowDownloaded");
	writer.addAttribute("type", "int");
	writer.addElementData(showDownloaded->get_active());
	writer.endElement();

	writer.startElement("ShowUploaded");
	writer.addAttribute("type", "int");
	writer.addElementData(showUploaded->get_active());
	writer.endElement();

	writer.startElement("ShowDSpeed");
	writer.addAttribute("type", "int");
	writer.addElementData(showDSpeed->get_active());
	writer.endElement();

	writer.startElement("ShowUSpeed");
	writer.addAttribute("type", "int");
	writer.addElementData(showUSpeed->get_active());
	writer.endElement();

	writer.startElement("ShowRatio");
	writer.addAttribute("type", "int");
	writer.addElementData(showRatio->get_active());
	writer.endElement();

	writer.startElement("ShowRest");
	writer.addAttribute("type", "int");
	writer.addElementData(showRest->get_active());
	writer.endElement();

	writer.startElement("ShowPercentage");
	writer.addAttribute("type", "int");
	writer.addElementData(showPercentage->get_active());
	writer.endElement();

	writer.startElement("ShowSeeders");
	writer.addAttribute("type", "int");
	writer.addElementData(showSeeders->get_active());
	writer.endElement();

	writer.startElement("ShowDownloaders");
	writer.addAttribute("type", "int");
	writer.addElementData(showDownloaders->get_active());
	writer.endElement();

	writer.startElement("ShowStatus");
	writer.addAttribute("type", "int");
	writer.addElementData(showStatus->get_active());
	writer.endElement();

	writer.startElement("ShowETA");
	writer.addAttribute("type", "int");
	writer.addElementData(showETA->get_active());
	writer.endElement();

	writer.endElement();
	writer.endDocument();
}

void TorrentsWindow::addRow(Torrent *t)
{
	int position;
	Gtk::TreeModel::Row row;

	position = refTreeModel->children().size() + 1;
	row = *(refTreeModel->append());

	// Health
	row[torrentsColumns.m_health] = BAD;
	row[torrentsColumns.m_col_health] = badHealth;

	updateRow(t, row);
	setMenuSensitivity();
}

void TorrentsWindow::updateRows()
{
	// Update rows
	int i;
	Gtk::TreeModel::Row row;
	Gtk::TreeModel::iterator j;

	TorrentsManagerPtr tm;
	tm = Core::get_torrents_manager();
	Torrent *t;

	for ( j=refTreeModel->children().begin(); j != refTreeModel->children().end(); ++j){
		int i;
		row = *j;
		i = row[torrentsColumns.m_col_id];

		if (torrentsWindowType == DOWNLOADING_TORRENTS)
			t = tm->get_downloading_torrent_at(i);
		else
			t = tm->get_shared_torrent_at(i);

		updateRow(t, row);
	}
}

bool TorrentsWindow::isSorted()
{
	Gtk::TreeView::Column* pColumn;
	for(int i = 1; i < 14; i++)
	{
		pColumn = torrentsTreeView.get_column(i);
		if(pColumn->get_sort_indicator())
			return true;
	}

	return false;
}


void TorrentsWindow::removeGivenRow(int tId){
	Gtk::TreeModel::iterator i;
	Gtk::TreeModel::Row row;

	for ( i = refTreeModel->children().begin(); i != refTreeModel->children().end(); ++i)
	{
		row = *i;
		if (row[torrentsColumns.m_col_id] == tId )
		{
			i = refTreeModel->erase(*i);
			break;
		}
		if (row[torrentsColumns.m_col_id] > tId)
			row[torrentsColumns.m_col_id] = row[torrentsColumns.m_col_id] - 1;
	}
	while(i != refTreeModel->children().end())
	{
		row = *i;
		if (row[torrentsColumns.m_col_id] > tId)
			row[torrentsColumns.m_col_id] = row[torrentsColumns.m_col_id] - 1;
		++i;
	}

	/*menuBar->setTorrentSelected(false);*/
	setMenuSensitivity();
}

int TorrentsWindow::getSelectedRow()
{
	std::vector<Gtk::TreePath> temp_list;

	if(refTreeSelection->count_selected_rows()){ //If anything is selected
		temp_list = refTreeSelection->get_selected_rows();
		return (temp_list.begin()->front() + 1);
	}

	return 0;
}

void TorrentsWindow::setSelectedRow(int tId)
{
	std::ostringstream rowpos;
    rowpos << tId - 1;
    torrentsTreeView.set_cursor(Gtk::TreePath(rowpos.str()));
}

void TorrentsWindow::unselectRows(void)
{
	refTreeSelection->unselect_all();
}

void TorrentsWindow::updateRow(Torrent *t, Gtk::TreeModel::Row &row)
{
	std::ostringstream seeders, downloaders, ratio;

	seeders << t->get_seeders();
	if ( t->get_total_seeders() != -1)
		seeders << "(" << t->get_total_seeders() << ")";
	downloaders << t->get_downloaders();
	if ( t->get_total_downloaders() != -1)
		downloaders << "(" << t->get_total_downloaders() << ")";

	// Numeric sort rows.
	row[torrentsColumns.m_percentage] = t->get_percentage();
	row[torrentsColumns.m_size] = t->get_size();
	row[torrentsColumns.m_downloaded] = t->get_downloaded();
	row[torrentsColumns.m_uploaded] = t->total_uploaded();
	row[torrentsColumns.m_rest] = t->get_remaining();
	row[torrentsColumns.m_down_speed] = t->get_download_speed();
	row[torrentsColumns.m_up_speed] = t->get_upload_speed();
	row[torrentsColumns.m_time] = t->get_eta();
	row[torrentsColumns.m_ratio] = t->get_ratio();

	// Health row
	Health hth;
	if ( (t->get_peers() || t->get_seeders() ) && t->has_incoming() &&
			t->tracker_connected() )
		hth = GOOD;
	else if ( (t->get_peers() || t->get_seeders() ) && t->tracker_connected())
		hth = FIREWALLED;
	else if ( t->get_peers() || t->get_seeders() )
		hth = NOTRACKER;
	else
		hth = BAD;

	if (row[torrentsColumns.m_health] != hth)
	{
		row[torrentsColumns.m_health] = hth;
		if (hth == GOOD)
			row[torrentsColumns.m_col_health] = goodHealth;
		if (hth == FIREWALLED)
			row[torrentsColumns.m_col_health] = firewalledHealth;
		if (hth == NOTRACKER)
			row[torrentsColumns.m_col_health] = noTrackerHealth;
		if (hth == BAD)
			row[torrentsColumns.m_col_health] = badHealth;
	}

	// Data rows.
	row[torrentsColumns.m_col_id] = t->position();
	row[torrentsColumns.m_col_name] = t->name();
	row[torrentsColumns.m_col_size] = size_to_string( t->get_size() );
	row[torrentsColumns.m_col_downloaded] = size_to_string( t->get_downloaded() );
	row[torrentsColumns.m_col_uploaded] = size_to_string( t->total_uploaded() );

	ratio << std::setiosflags(std::ios::fixed | std::ios::showpoint) <<
		std::setprecision(2) << row[torrentsColumns.m_ratio];
	row[torrentsColumns.m_col_ratio] = ratio.str();

	row[torrentsColumns.m_col_rest] = size_to_string( t->get_remaining() );
	row[torrentsColumns.m_col_seeders] = seeders.str();
	row[torrentsColumns.m_col_downloaders] = downloaders.str();
	row[torrentsColumns.m_col_download_speed] = size_to_string( t->get_download_speed() );
	row[torrentsColumns.m_col_upload_speed] = size_to_string( t->get_upload_speed() );

	row[torrentsColumns.m_col_percentage] = t->get_percentage();
	row[torrentsColumns.m_col_status] = t->get_status_string();

	if (row[torrentsColumns.m_time] == FLT_MAX)
		row[torrentsColumns.m_col_time] = _("Unknown");
	else
		row[torrentsColumns.m_col_time] =
			time_to_string(static_cast<boost::int64_t>(row[torrentsColumns.m_time]));
}

void TorrentsWindow::hideColumn(int id)
{
	Gtk::TreeView::Column* pColumn = torrentsTreeView.get_column(id);

	if (pColumn->get_visible()) pColumn->set_visible(false);
	else pColumn->set_visible(true);
}

void TorrentsWindow::mouse_button_press_event(GdkEventButton* event)
{
	if( (event->type == GDK_BUTTON_PRESS) && (event->button == 3) && (torrentsSelected() == 1) )
	{
		if(rm_pMenuPopup)
			rm_pMenuPopup->popup(event->button, event->time);
	}
	else if( (event->type == GDK_BUTTON_PRESS) && (event->button == 1) && !isCtrlKeyPressed)
	{
		menuBar->setTorrentSelected(false);
		unselectRows();
	}
	else if( (event->type == GDK_BUTTON_PRESS) && (event->button == 3) && !torrentsSelected() )
	{
		menuBar->setTorrentSelected(false);
		unselectRows();
		if (nsm_pMenuPopup)
			nsm_pMenuPopup->popup(event->button, event->time);
	}
}

bool TorrentsWindow::getSelectedTorrent(int &position)
{
	std::vector<Gtk::TreePath> torrents = refTreeSelection->get_selected_rows();

	if (torrents.empty())
	{
		position = 0;
		return false;
	}

	Gtk::TreeModel::Row row = *(torrentsTreeView.get_model()->get_iter(*torrents.begin()));

	position = row[torrentsColumns.m_col_id];

	return true;
}

bool TorrentsWindow::getSelectedTorrents(std::vector<int> &positions)
{
	std::vector<Gtk::TreePath> torrents = refTreeSelection->get_selected_rows();
	std::vector<Gtk::TreePath>::iterator iter;

	if (torrents.empty()) return false;

	for (iter = torrents.begin(); iter != torrents.end(); ++iter)
	{
		Gtk::TreeModel::Row row = *(torrentsTreeView.get_model()->get_iter(*iter));
		positions.push_back(row[torrentsColumns.m_col_id]);
	}

	return true;
}

void TorrentsWindow::setMenuSensitivity()
{
	if (torrentsSelected() == 0)
		menuBar->setTorrentSelected(false);
	else if (torrentsSelected() == 1)
		menuBar->setTorrentSelected();
	else
		menuBar->setTorrentsSelected();
}

int TorrentsWindow::torrentsSelected()
{
	return (refTreeSelection->count_selected_rows());
}

void TorrentsWindow::key_press_event(GdkEventKey* event)
{
	if (event->keyval == GDK_Control_L || event->keyval == GDK_Control_R)
		isCtrlKeyPressed = true;

}

void TorrentsWindow::key_release_event(GdkEventKey* event)
{
	if (event->keyval == GDK_Control_L || event->keyval == GDK_Control_R)
		isCtrlKeyPressed = false;
}

bool TorrentsWindow::focus_in_event(GdkEventFocus*)
{
	torrentsTab->checkTorrentsSelection(torrentsWindowType);

	isCtrlKeyPressed = false;

	setMenuSensitivity();
}
